﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Reflection;
using System.Text.RegularExpressions;

namespace Languages
{
    public static class Translator
    {
        private static string LanguageResourceName = "Languages.RLTrans_ISO639-1.xml";               

        private static Dictionary<string, Dictionary<string, string>> dictionary;

        static Translator()
        {
            var ass = Assembly.GetExecutingAssembly();
            var langsStream = ass.GetManifestResourceStream(LanguageResourceName);
            var langs = new XmlDocument();
            langs.Load(langsStream);
            InitDictionary(langs);
        }

        private static void InitDictionary(XmlDocument doc)
        {
            var langs = doc.SelectNodes("/TurnTexts/Language");

            dictionary = new Dictionary<string, Dictionary<string, string>>(langs.Count);

            for (int i = 0; i < langs.Count; i++)
            {
                var lang = langs[i];
                var langCode = lang.Attributes["Name"].InnerText;

                var words = new Dictionary<string, string>();
                dictionary.Add(langCode, words);

                for (int j = 0; j < lang.ChildNodes.Count; j++)
                {
                    var child = lang.ChildNodes[j];
                    for (int k = 0; k < child.Attributes.Count; k++)
                    {
                        var attr = child.Attributes[k];
                        words.Add(child.Name + "." + attr.Name, attr.InnerText);
                    }
                }
            }            
        }

        public static IList<Maneuver> Translate(IList<Maneuver> maneuvers, string toLanguage)
        {            
            Dictionary<string, string> dict;
            if (dictionary.TryGetValue(toLanguage, out dict))
            {
                var result = new List<Maneuver>(maneuvers.Count);
                var evaluator = new MatchEvaluator(m => Translate(m.Value, dict));
                foreach (var m in maneuvers)
                {
                    var translatedManeuver = new Maneuver();
                    var output = Regex.Replace(m.Instruction, "({[^}]*})", evaluator);
                    var cleanOutput = Regex.Replace(output, @"\s+", " ").Replace("}", "");
                    translatedManeuver.Instruction = cleanOutput;
                    translatedManeuver.SucceedingSegmentIndex = m.SucceedingSegmentIndex;
                    translatedManeuver.Type = m.Type;
                    result.Add(translatedManeuver);
                }
                return result;
            } else
            {
                return null;
            }
        }

        public static string Translate(string maneuverItem, Dictionary<string, string> dict)
        {
            maneuverItem = maneuverItem.Replace("{", "");
            maneuverItem = maneuverItem.Replace("}", "");
            var rules = maneuverItem.Split(new char[] {'|'} );
            if (rules.Length == 1)
            {
                return dict[rules[0]];
            }
            else
            {
                var result = dict[rules[0]];
                for (int i = 1; i < rules.Length; i++)
                {
                    var ruleSpec = rules[i].Split(new char[] { '=' });
                    result = result.Replace("%" + ruleSpec[0], dict[ruleSpec[1]]);
                }
                return result;
                
            }                
        }               

    }
}
